/******************************************************************************
 *
 * Copyright (c) 2000 by Mindbright Technology AB, Stockholm, Sweden.
 *                 www.mindbright.se, info@mindbright.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *****************************************************************************
 * $Author: mats $
 * $Date: 2000/03/20 08:35:27 $
 * $Name: rel1-2 $
 *****************************************************************************/
package mindbright.security;

import java.security.NoSuchAlgorithmException;

import java.math.BigInteger;

/**
 * Class implementing message authentication through keyed hashing
 * (referred to as HMAC) described in rfc2104
 */
public abstract class HMAC {

    protected int macLength;
    protected int blockSize;

    public static HMAC getInstance(String algorithmSpec) throws NoSuchAlgorithmException {
	if(algorithmSpec.startsWith(RFC2104HMAC.SPEC_PREFIX)) {
	    return new RFC2104HMAC(algorithmSpec);
	} else {
	    return new SimpleHMAC(algorithmSpec);
	}
    }

    public abstract void init(byte[] key);

    public final byte[] digest(byte[] data) {
	return digest(data, 0, data.length);
    }

    public final int digest(byte[] data, int srcOff, int srcLen, byte[] dest, int destOff) {
	byte[] mac = digest(data, srcOff, srcLen);
	System.arraycopy(mac, 0, dest, destOff, mac.length);
	return mac.length;
    }

    public abstract byte[] digest(byte[] data, int off, int len);

    final public int getMacLength() {
	return macLength;
    }

    final public int blockSize() {
	return blockSize;
    }

    protected String parseAlgorithmSpec(String algorithmSpec) {
	algorithmSpec = algorithmSpec.toLowerCase();
	int i = algorithmSpec.indexOf('-');
	String alg;
	if(i != -1) {
	    alg       = algorithmSpec.substring(0, i);
	    macLength = Integer.parseInt(algorithmSpec.substring(i + 1)) / 8;
	} else {
	    alg       = algorithmSpec;
	    macLength = -1;
	}

	return alg.toUpperCase();
    }

    // !!! DEBUG
    public static void main(String[] argv) {
	try {
	    byte[] key = { (byte)0x0b, (byte)0x0b, (byte)0x0b, (byte)0x0b,
			   (byte)0x0b, (byte)0x0b, (byte)0x0b, (byte)0x0b,
			   (byte)0x0b, (byte)0x0b, (byte)0x0b, (byte)0x0b,
			   (byte)0x0b, (byte)0x0b, (byte)0x0b, (byte)0x0b };

	    HMAC hmac = HMAC.getInstance("hmac-md5");
	    hmac.init(key);
	    byte[] data = "Hi There".getBytes();
	    data = hmac.digest(data);
	    System.out.println("digest = " + printHex(data));

	    key = "Jefe".getBytes();
	    hmac = HMAC.getInstance("hmac-md5");
	    hmac.init(key);
	    data = "what do ya want for nothing?".getBytes();
	    data = hmac.digest(data);
	    System.out.println("digest = " + printHex(data));

	    key = "Jefe".getBytes();
	    hmac = HMAC.getInstance("hmac-md5-96");
	    hmac.init(key);
	    data = "what do ya want for nothing?".getBytes();
	    data = hmac.digest(data);
	    System.out.println("digest = " + printHex(data));

	    key = "Jefe".getBytes();
	    hmac = HMAC.getInstance("md5-96");
	    hmac.init(key);
	    data = "what do ya want for nothing?".getBytes();
	    data = hmac.digest(data);
	    System.out.println("digest = " + printHex(data));

	    key = "Jefe".getBytes();
	    hmac = HMAC.getInstance("md5-8");
	    hmac.init(key);
	    data = "what do ya want for nothing?".getBytes();
	    data = hmac.digest(data);
	    System.out.println("digest = " + printHex(data));

	    key = "Jefe".getBytes();
	    hmac = HMAC.getInstance("hmac-sha1");
	    hmac.init(key);
	    data = "what do ya want for nothing?".getBytes();
	    data = hmac.digest(data);
	    System.out.println("digest = " + printHex(data));

	    key = "Jefe".getBytes();
	    hmac = HMAC.getInstance("hmac-sha1-96");
	    hmac.init(key);
	    data = "what do ya want for nothing?".getBytes();
	    data = hmac.digest(data);
	    System.out.println("digest = " + printHex(data));

	} catch (Exception e) {
	    e.printStackTrace();
	    System.out.println("Error: " + e);
	}
    }

    static String printHex(byte[] buf) {
	byte[] out = new byte[buf.length + 1];
	out[0] = 0;
	System.arraycopy(buf, 0, out, 1, buf.length);
	BigInteger big = new BigInteger(out);
	return big.toString(16);
    }

}
