/******************************************************************************
 *
 * Copyright (c) 2000 by Mindbright Technology AB, Stockholm, Sweden.
 *                 www.mindbright.se, info@mindbright.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *****************************************************************************
 * $Author: mats $
 * $Date: 2000/03/15 18:35:23 $
 * $Name: rel1-2 $
 *****************************************************************************/
package mindbright.security;

import java.security.NoSuchAlgorithmException;

/**
 * Class implementing message authentication through keyed hashing
 * (referred to as HMAC) described in rfc2104
 */
public class RFC2104HMAC extends HMAC {

    public final static String SPEC_PREFIX = "hmac-";

    byte[] k_ipad;
    byte[] k_opad;

    MessageDigest inner;
    MessageDigest outer;
    MessageDigest innerClone;
    MessageDigest outerClone;

    RFC2104HMAC(String algorithmSpec) throws NoSuchAlgorithmException {
	algorithmSpec = parseAlgorithmSpec(algorithmSpec.substring(SPEC_PREFIX.length()));

	try {
	    inner = MessageDigest.getInstance(algorithmSpec);
	    outer = MessageDigest.getInstance(algorithmSpec);
	} catch(InstantiationException ie) {
	    throw new NoSuchAlgorithmException();
	} catch(IllegalAccessException ia) {
	    throw new NoSuchAlgorithmException();
	} catch(ClassNotFoundException ce) {
	    throw new NoSuchAlgorithmException();
	}

	if(macLength == -1) {
	    macLength = inner.hashSize();
	}

	blockSize = inner.blockSize();

	k_ipad = new byte[blockSize];
	k_opad = new byte[blockSize];
    }

    public void init(byte[] key) {
	inner.reset();
	outer.reset();

	if(key.length > blockSize) {
	    inner.update(key);
	    key = inner.digest();
	    inner.reset();
	}

	System.arraycopy(key, 0, k_ipad, 0, key.length);
	System.arraycopy(key, 0, k_opad, 0, key.length);

	for(int i = 0; i < blockSize; i++) {
	    k_ipad[i] ^= (byte)0x36;
	    k_opad[i] ^= (byte)0x5c;
	}

	updatePads();

	if(inner instanceof Cloneable) {
	    try {
		innerClone = (MessageDigest)inner.clone();
		outerClone = (MessageDigest)outer.clone();
	    } catch (CloneNotSupportedException e) {
		innerClone = null;
		outerClone = null;
	    }
	}
    }

    public byte[] digest(byte[] data, int off, int len) {
	byte[] innerHash, outerHash, hmac;
	updatePads();
	inner.update(data, off, len);
	innerHash = inner.digest();
	outer.update(innerHash);
	outerHash = outer.digest();

	hmac = new byte[macLength];
	System.arraycopy(outerHash, 0, hmac, 0, macLength);

	return hmac;
    }

    private void updatePads() {
	if(innerClone != null) {
	    try {
		inner = (MessageDigest)innerClone.clone();
		outer = (MessageDigest)outerClone.clone();
	    } catch (CloneNotSupportedException e) {
		innerClone = null;
		outerClone = null;
		updatePads();
	    }
	} else {
	    inner.reset();
	    outer.reset();
	    inner.update(k_ipad);
	    outer.update(k_opad);
	}
    }

}
